#!/bin/bash

check_env() {
  #First the agreement message.
  if ${CARRIER} ; then
    log_info "Welcome to Sipwise C5 CARRIER installer"
  elif ${PRO} ; then
    log_info "Welcome to Sipwise C5 PRO installer"
  else
    log_info "Welcome to Sipwise C5 CE installer."
    log_info
    log_info "This software comes WITH NO WARRANTY."
    log_info "If you need SLA please consider upgrading to Sipwise C5 PRO."
    log_info
    # variable INTERACTIVE is set in deployment.sh so if ngcp-installer was run from deployment.sh
    # no need to print/ask about the shared system
    if ! "${DEPLOYMENT_SH}"; then
      log_info "This installation script is not intended to run in a shared system,"
      log_info "as it can add/delete/update existing configurations."
      log_info "Please run this script only in a base install of 64 bit Debian ${DEBIAN_VERSION} (${DEBIAN_CODENAME})"
      log_info "on a dedicated server."
      log_info
      log_info_no_nl "Do you want to continue with the installation process? (y/N): "
      if [ "$FORCE" == "yes" ]; then
        log_info
        log_info "Forcing as requested via installer option FORCE=yes"
      else
        read AGREE
        if [ "$AGREE" != "y" ]; then
          exit 0
        fi
      fi
    fi
  fi

  #Check if I'm root
  if [[ $EUID -ne 0 ]]; then
    log_error "This script must run as root"
    exit 1
  else
    log_info "Running as root... OK"
  fi

  if ! [ -e /etc/hostname ] ; then
    # ensure that hostname isn't empty
    if [ -z "$(hostname)" ] ; then
      log_error "file /etc/hostname doesn't exist and output of 'hostname' is empty."
      exit 1
    fi

    # e.g. on EC2 systems hostname returns something useful, though /etc/hostname is missing
    log_info "/etc/hostname does not exist yet, creating using '$(hostname)'."
    hostname > /etc/hostname
  fi

  # MT#17339 ensure /etc/hostname isn't empty
  if ! [ -s /etc/hostname ] ; then
    log_error "file /etc/hostname exists but is empty, please set a valid hostname."
    log_info "Exiting now to avoid possible installation problems."
    exit 1
  fi

  if ! [ -r /proc/meminfo ]; then
    log_error "Can not read /proc/meminfo - please make sure /proc is mounted."
    exit 1
  fi

  # warn when running inside OpenVZ container
  if [ -d /proc/vz/vzaquota ] && ! [ -f /proc/vz/version ] ; then
    log_warn "#################################################################"
    log_warn "Warning: trying to run installer inside OpenVZ container?"
    log_warn "This is not supported at the moment and might fail."
    log_warn "#################################################################"
    log_warn "Will continue in 10 seconds..."
    sleep 10
  fi

  # warn when running inside lxc container
  if [ -r /proc/1/environ ] && tr '\000' '\n' < /proc/1/environ | grep -Eiq '^container='; then
    log_warn "#################################################################"
    log_warn "Warning: trying to run installer inside lxc?"
    log_warn "This is not supported at the moment and might fail."
    log_warn "#################################################################"
    log_warn "Will continue in 10 seconds..."
    sleep 10
  fi

  MEM=$(awk '/^MemTotal:/ {print $2}' /proc/meminfo)
  if [ "${MEM}" -lt 1023000 ]; then
    log_warn "#################################################################"
    log_warn "Warning: less than 1GB of RAM, system might not work as expected!"
    log_warn "#################################################################"
    log_warn "Will continue in 10 seconds..."
    sleep 10
  fi

  if [ -r /var/lib/apt/cdroms.list ] && [ -r /etc/apt/sources.list ] ; then
    log_warn "Looks like this system was installed using a Debian CD."
    log_warn "Please notice that Sipwise recommends usage of either"
    log_warn "the NGCP install CD or the Netinstall ISO as installation medium."

    log_info "Checking for enabled cdrom entries in /etc/apt/sources.list:"

    if ! grep -q '^deb cdrom:' /etc/apt/sources.list ; then
      log_info "nothing found, apt-get should work as needed."
    else
      log_warn "#################################################################"
      log_warn "Warning: CD-ROM entries for apt-get identified."
      log_warn "#################################################################"
      log_warn "Identified the following entries:"

      grep '^deb cdrom:' /etc/apt/sources.list
      log_info

      log_warn "If running ngcp-installer seems to hang on your system please"
      log_warn "disable the cdrom entries via executing:"
      log_warn "  sed -i 's/^deb cdrom:/# &/' /etc/apt/sources.list"
      log_warn "  apt-get update"
      log_warn "#################################################################"
      log_warn "Will continue in 10 seconds..."
      sleep 10
    fi
  fi

  # Confirm systemd in use on CE/PRO/Carrier
  if [ "$(dpkg-query -f "\${db:Status-Status} \${db:Status-Eflag}" -W systemd-sysv 2>/dev/null)" != 'installed ok' ]
  then
    log_warn
    log_warn
    log_warn "The current NGCP release does support 'systemd' init system ONLY,"
    log_warn "but it does not seem to be installed."
    log_warn
    log_warn "The current init system will be replaced with 'systemd'!"
    log_warn
    log_warn "PLEASE ABORT THE INSTALLATION IF YOU DEPEND ON THE CURRENT INIT SYSTEM!"
    log_warn
    log_warn "NOTE: modern Debian releases use 'systemd' by default."

    log_info
    log_info_no_nl "Do you want to proceed with ngcp-installer and install 'systemd'? (y/N) "

    if [ "$FORCE" == "yes" ]; then
      log_info
      log_warn "Forcing as requested via installer option FORCE=yes"
    else
      unset AGREE
      read AGREE
      if [ "$AGREE" != "y" ]; then
        log_info "Aborted ngcp-installer as requested."
        exit 0
      fi
    fi
  fi

  # Here is the list of required packages so we check if they are installed
  # and list of required minimal version. If the version does not matter - leave an empty value ''
  declare -A packages
  packages=(
    ['ca-certificates']=''
    ['jq']=''
    ['wget']=''
  )

  local run_apt_update=false
  for package in "${!packages[@]}" ; do
    declare -a package_info
    # shellcheck disable=SC2016
    package_info=( $(dpkg-query --show --showformat='${db:Status-Status} ${db:Status-Eflag} ${Version}' "${package}" || true) )

    if [[ "${package_info[0]} ${package_info[1]}" != 'installed ok' ]]; then
      log_warn "missing package '${package}' detected, going to install it."
      run_apt_update=true
    elif dpkg --compare-versions "${package_info[2]}" lt "${packages[${package}]}" ; then
      log_warn "version of '${package}' is too old, going to update it."
      run_apt_update=true
    fi
  done

  if "${run_apt_update}"; then
    log_info "Making sure Debian's database is up2date:"
    env "${APT_ENV[@]}" apt-get "${APT_PARAMS[@]}" update

    log_info "Installing ${!packages[*]} packages:"
    if ! env "${APT_ENV[@]}" apt-get "${APT_PARAMS[@]}" install "${!packages[@]}" ; then
      log_error "Failed to install packages (${!packages[*]})."
      exit 1
    fi
  fi

  # Check if we are running the according Debian version
  if [[ "${DEBIAN_CODENAME}" == "${NGCP_DEBIAN_CURRENT_CODENAME}" ]]; then
    log_info "Check Debian version... OK"
  elif [[ "${DEBIAN_CODENAME}" == "${NGCP_DEBIAN_NEXT_CODENAME}" ]] && grep -q '/sid' /etc/debian_version ; then
    if [ "$FORCE" == "yes" ]; then
      log_info "Check Debian version... running on Debian/${DEBIAN_CODENAME} testing release."
      log_warn "Forcing as requested via installer option FORCE=yes"
    else
      log_warn "#################################################################"
      log_warn "Warning: trying to run installer on Debian/${DEBIAN_CODENAME} release."
      log_warn "This is not supported at the moment and might fail."
      log_warn "#################################################################"
      log_warn "Will continue in 10 seconds..."
      sleep 10
    fi
  else
    log_error "Cannot detect the version of Debian, or is a version not supported."
    log_error "Please run the installer in a Debian ${NGCP_DEBIAN_CURRENT_VERSION} (${NGCP_DEBIAN_CURRENT_CODENAME}) system."
    exit 1
  fi

  check_sipwise_repo_connectivity

  check_sipwise_gpg_key

  #Check if this is a clean install
  if [ -f /etc/sipwise_ngcp_version ]; then
    log_error "This script should not run on an existing Sipwise C5 installation."
    exit 1
  else
    log_info "Looks like a clean install... OK"
  fi

  #Check that we're running amd64
  RUNNING_ARCH=$(uname -m)
  if [ "$RUNNING_ARCH" != "x86_64" ]; then
    log_error "The installer should run in a 64 bit environment. Make sure you installed amd64 version of Debian"
    exit 1
    #TODO Let the user continue
  else
    log_info "Check 64 bit capable system... OK"
  fi

  #Check /var/lib/mysql exists
  if ! "${UPGRADE_MODE}"; then
    if [[ -d /var/lib/mysql || -d /ngcp-data/mysql ]]; then
      log_warn "An existing installation of MariaDB has been detected in /var/lib/mysql and/or /ngcp-data/mysql."
      log_warn "You should only run this installer in a clean install of Debian ${DEBIAN_VERSION} (${DEBIAN_CODENAME}) amd64."
      log_warn "Please stop MariaDB, backup it and remove DB folders manually to continue with the installation here."
      exit 1
    fi
  fi
}

check_sipwise_repo_connectivity() {
  log_info "Testing web access..."

  local SIPWISE_REPO="${SIPWISE_REPO_TYPE}/${VERSION}"
  local CODENAME="${DEBIAN_CODENAME}"
  if [[ "${VERSION}" == 'trunk' ]]; then
    SIPWISE_REPO="autobuild"
    CODENAME="release-trunk-${DEBIAN_CODENAME}"
  elif [[ "${VERSION}" == 'trunk-weekly' ]] ; then
    SIPWISE_REPO="autobuild/release/release-trunk-weekly"
    CODENAME="release-trunk-weekly"
  fi
  local check_path="${SIPWISE_URL}/${SIPWISE_REPO}/dists/${CODENAME}/InRelease"

  local timeout=10
  while [[ "${timeout}" -gt 0 ]]; do
    if wget -q -T 10 --retry-connrefused --tries=3 -O /dev/null "${check_path}"; then
      log_info "Web test OK"
      return 0
    fi
    log_info "Can't connect to sipwise repo, waiting for ${timeout} seconds..."
    sleep 1
    timeout=$(( timeout - 1 ))
  done

  log_error "Can not connect to ${check_path}"
  exit 1
}

check_running_on_azure() {
  log_info "* Checking for Azure environment"

  if ! systemd-detect-virt --vm | grep -q 'microsoft' ; then
    return 1
  fi

  log_info "* Looks like a Microsoft VM, checking for Azure environment"

  local env_info=$(wget --timeout=3 --wait=3 --tries=1 --quiet --no-proxy --header="Metadata:true" -O- "http://169.254.169.254/metadata/instance?api-version=2021-02-01" | jq -r '.compute.azEnvironment')
  if [[ "${env_info}" == "AzurePublicCloud" ]] ; then
    log_info "* Looks like an Azure system"
    return 0
  fi

  log_info "* Does not look like an Azure system"
  return 1
}

check_running_on_aws() {
  log_info "* Checking for AWS environment"

  if [ -d /proc/xen ] ; then
    if wget --timeout=3 --wait=3 --tries=1 -O /dev/null --quiet http://169.254.169.254/2008-02-01 ; then
      log_info "* Looks like an older Xen based Amazon EC2 system"
    else
      log_info "* Does not look like an Amazon EC2 system (xen/wget check)"
      return 1
    fi
  elif grep -q 'Amazon EC2' /sys/class/dmi/id/bios_vendor ; then
    log_info "* Looks like an Amazon EC2 system (bios_vendor check)"
  else
    log_info "* Does not look like an Amazon EC2 system"
    return 1
  fi
}

# This checks that looks like an IPv4 address.  Not perfect, it accepts >255 and
# things like 000, but it'll probably catch most problems in real life.
is_valid_ipv4() {
  local ipv4="$1"

  if [[ -z "${ipv4}" ]] ; then
    log_warn "Empty IPv4 address"
    return 1
  elif [[ "${ipv4}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]; then
    log_info "Looks like a valid IPv4 address: '${ipv4}'"
    return 0
  else
    log_warn "It does not look like a valid IPv4 address: '${ipv4}'"
    return 1
  fi
}

check_running_inside_virt_env() {
  if ! command -v ngcp-virt-identify &>/dev/null ; then
    log_warn "ngcp-virt-identify not available, skipping checks"
    return 0
  fi

  if ngcp-virt-identify ; then
    log_debug "ngcp-virt-identify reported virtualized environment"
    return 0
  fi

  log_debug "ngcp-virt-identify reported non-virtualized environment"
  return 1
}

check_terminal_multiplexer() {
  "${DOCKER_BUILD}" && return
  if [[ -x "/usr/sbin/ngcp-screen-check" ]]; then
    ngcp-screen-check
  else
    log_die "Script /usr/sbin/ngcp-screen-check was not found. Check if Sipwise C5 is fully installed."
    return 1
  fi
}

check_sipwise_gpg_key() {
  local sipwise_key='/etc/apt/trusted.gpg.d/sipwise-keyring-bootstrap.gpg'
  if [[ -f "${sipwise_key}" ]]; then
    log_info "File '${sipwise_key}' was found"
    return
  fi
  log_info "There is no '${sipwise_key}' file"

  local status
  status="$(dpkg-query -f "\${db:Status-Status} \${db:Status-Eflag}" -W ngcp-archive-keyring 2>/dev/null || true)"
  if [[ "${status}" == 'installed ok' ]]; then
    log_info "Package ngcp-archive-keyring is installed"
    return
  fi
  log_info "There is no 'ngcp-archive-keyring' package"

  local key_path="${SIPWISE_URL}/spce/keyring/sipwise-keyring-bootstrap.gpg"

  local tmp_key
  tmp_key="$(mktemp -t ngcp-installer-sipkey-XXXXXXXXXX)"
  log_info "Downloading ${key_path} into ${tmp_key}"
  wget -q -T 10 --retry-connrefused --tries=3 -O "${tmp_key}" "${key_path}"
  chmod 644 "${tmp_key}"
  local sipwise_key_checksum
  sipwise_key_checksum=$(sha256sum "${tmp_key}" | awk '{print $1}')

  log_info "sha256sum checksum of the file is ${sipwise_key_checksum}"
  system_ask_user "Is it correct and should be imported into the system?"
  log_info "The key has been accepted, installing it as ${sipwise_key}"
  mv "${tmp_key}" "${sipwise_key}"
}

check_already_configured() {
  if [[ -f /etc/sipwise_ngcp_version ]]; then
    if grep 'System configured' -q /etc/sipwise_ngcp_version ; then
      system_ask_user "System already configured and ready. Do you want to rerun configuration?"
    fi
  fi
}

# vim: ft=sh
